@@ -17,7 +17,7 @@
 
 (in-package #:clsql-tests)
 
-#.(clsql:locally-enable-sql-reader-syntax)
+(clsql-sys:file-enable-sql-reader-syntax)
 
 
 (def-view-class big ()
@@ -107,10 +107,10 @@
     ;; and stick a value in there.
     (progn (clsql-sys:create-view-from-class 'big)
 	   (values
-	     (clsql:table-exists-p [big] :owner *test-database-user*)
+	     (clsql:table-exists-p [big] )
 	     (progn
 	       (clsql:drop-table [big] :if-does-not-exist :ignore)
-	       (clsql:table-exists-p [big] :owner *test-database-user*)))
+	       (clsql:table-exists-p [big] )))
 	   )
   t nil)
 
@@ -184,5 +184,4 @@
 
 ))
 
-#.(clsql:restore-sql-reader-syntax-state)
 
