@@ -15,7 +15,8 @@
 
 (in-package #:clsql-tests)
 
-#.(clsql:locally-enable-sql-reader-syntax)
+(clsql-sys:file-enable-sql-reader-syntax)
+
 
 (setq *rt-oodml*
       '(
@@ -1155,6 +1156,3 @@
   t)
 ))
 
-
-
-#.(clsql:restore-sql-reader-syntax-state)
